@@ -954,7 +954,7 @@ def send_bugtracker(msg):
                     and 'roundup.cgi.exceptions.Unauthorised' in
                     exception.faultString):
                 message(_('Connection error!\nBad username or password!'))
-                return send_bugtracker(msg, parent)
+                return send_bugtracker(msg)
             tb_s = reduce(lambda x, y: x + y,
                     traceback.format_exception(sys.exc_type,
                         sys.exc_value, sys.exc_traceback))
@@ -1461,11 +1461,17 @@ def safe_eval(source, data=None):
         'dict': dict,
         }}, data)
 
-def timezoned_date(date):
-    if pytz and rpc.CONTEXT.get('timezone'):
+def timezoned_date(date, reverse=False):
+    if pytz and rpc.CONTEXT.get('timezone') and rpc.TIMEZONE:
         lzone = pytz.timezone(rpc.CONTEXT['timezone'])
         szone = pytz.timezone(rpc.TIMEZONE)
+        if reverse:
+            lzone, szone = szone, lzone
         sdt = szone.localize(date, is_dst=True)
         ldt = sdt.astimezone(lzone)
         date = ldt
     return date
+
+
+def untimezoned_date(date):
+    return timezoned_date(date, reverse=True).replace(tzinfo=None)
