@@ -5,13 +5,14 @@ import gtk
 import gettext
 import os
 from tryton.config import CONFIG, TRYTON_ICON, PIXMAPS_DIR
+from tryton.common import get_toplevel_window
 
 _ = gettext.gettext
 
 
 class Tips(object):
 
-    def __init__(self, parent):
+    def __init__(self):
         self.tips = [
             _('''<b>Welcome to Tryton</b>
 
@@ -33,10 +34,15 @@ other known moons. It is also more massive than all the Solar System's
 
 You can copy records from any list with Ctrl + C
 and paste in any application with Ctrl + V
+'''),
+            _('''<b>Export graphs</b>
+
+You can save any graphs in PNG file with right-click on it.
 '''),
         ]
 
 
+        parent = get_toplevel_window()
         self.win = gtk.Dialog(_('Tips'), parent,
                 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
         self.win.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
@@ -88,7 +94,7 @@ and paste in any application with Ctrl + V
 
         try:
             self.number = int(CONFIG['tip.position'])
-        except:
+        except ValueError:
             self.number = 0
 
         self.tip_set()
