@@ -5,6 +5,7 @@ import gobject
 import gettext
 import tryton.common as common
 import tryton.rpc as rpc
+from tryton.exceptions import TrytonServerError
 import types
 from tryton.config import TRYTON_ICON
 import csv
@@ -17,10 +18,11 @@ _ = gettext.gettext
 class WinExport(object):
     "Window export"
 
-    def __init__(self, model, ids, parent, context=None):
+    def __init__(self, model, ids, context=None):
+        self.parent = common.get_toplevel_window()
         self.dialog = gtk.Dialog(
                 title= _("Export to CSV"),
-                parent=parent,
+                parent=self.parent,
                 flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT
                 | gtk.WIN_POS_CENTER_ON_PARENT)
         self.dialog.set_icon(TRYTON_ICON)
@@ -161,8 +163,6 @@ class WinExport(object):
         self.fields_data = {}
         self.context = context
 
-        self.parent = parent
-
         self.view1 = gtk.TreeView()
         self.view1.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
         self.view1.connect('row-expanded', self.on_row_expanded)
@@ -239,8 +239,8 @@ class WinExport(object):
         args = ('model', model, 'fields_get', None, rpc.CONTEXT)
         try:
             return rpc.execute(*args)
-        except Exception, exception:
-            return common.process_exception(exception, self.parent, *args)
+        except TrytonServerError, exception:
+            return common.process_exception(exception, *args)
 
     def on_row_expanded(self, treeview, iter, path):
         child = self.model1.iter_children(iter)
@@ -277,16 +277,15 @@ class WinExport(object):
                 [('resource', '=', self.model)], 0, None, None, rpc.CONTEXT)
         try:
             export_ids = rpc.execute(*args)
-        except Exception, exception:
-            export_ids = common.process_exception(exception, self.parent,
-                    *args)
+        except TrytonServerError, exception:
+            export_ids = common.process_exception(exception, *args)
             if not export_ids:
                 return
         args = ('model', 'ir.export', 'read', export_ids, None, rpc.CONTEXT)
         try:
             exports = rpc.execute(*args)
-        except Exception, exception:
-            exports = common.process_exception(exception, self.parent, *args)
+        except TrytonServerError, exception:
+            exports = common.process_exception(exception, *args)
             if not exports:
                 return
         args = ('model', 'ir.export.line', 'read',
@@ -294,8 +293,8 @@ class WinExport(object):
                 rpc.CONTEXT)
         try:
             lines = rpc.execute(*args)
-        except Exception, exception:
-            lines = common.process_exception(exception, self.parent, *args)
+        except TrytonServerError, exception:
+            lines = common.process_exception(exception, *args)
             if not lines:
                 return
         id2lines = {}
@@ -309,7 +308,7 @@ class WinExport(object):
         self.pref_export.set_model(self.predef_model)
 
     def add_predef(self, widget):
-        name = common.ask(_('What is the name of this export?'), self.parent)
+        name = common.ask(_('What is the name of this export?'))
         if not name:
             return
         iter = self.model2.get_iter_root()
@@ -327,8 +326,8 @@ class WinExport(object):
             }, rpc.CONTEXT)
         try:
             new_id = rpc.execute(*args)
-        except Exception, exception:
-            new_ids = common.process_exception(exception, self.dialog, *args)
+        except TrytonServerError, exception:
+            new_id = common.process_exception(exception, *args)
             if not new_id:
                 return
         self.predef_model.append((
@@ -348,8 +347,8 @@ class WinExport(object):
         args = ('model', 'ir.export', 'delete', export_id, rpc.CONTEXT)
         try:
             rpc.execute(*args)
-        except Exception, exception:
-            if not common.process_exception(exception, self.dialog, *args):
+        except TrytonServerError, exception:
+            if not common.process_exception(exception, *args):
                 return
         for i in range(len(self.predef_model)):
             if self.predef_model[i][0] == export_id:
@@ -369,9 +368,12 @@ class WinExport(object):
                                 (prefix + parent):
                             self.on_row_expanded(self.view1, iter,
                                     self.model1.get_path(iter))
+                            iter = self.model1.iter_children(iter)
+                            prefix = parent + '/'
                             break
-                        iter = self.model1.iter_next(iter)
-                    prefix = parent + '/'
+                        else:
+                            iter = self.model1.iter_next(iter)
+
             if field not in self.fields_data:
                 continue
             self.model2.append((self.fields_data[field]['string'], field))
@@ -397,10 +399,9 @@ class WinExport(object):
                 self.export_csv(fname, fields2, result,
                         self.wid_write_field_names.get_active(), popup=False)
                 os.close(fileno)
-                common.file_open(fname, 'csv', self.parent)
+                common.file_open(fname, 'csv')
             else:
                 fname = common.file_selection(_('Save As...'),
-                        parent=self.parent,
                         action=gtk.FILE_CHOOSER_ACTION_SAVE)
                 if fname:
                     self.export_csv(fname, fields2, result,
@@ -428,15 +429,13 @@ class WinExport(object):
             file_p.close()
             if popup:
                 if len(result) == 1:
-                    common.message(_('%d record saved!') % len(result),
-                            self.parent)
+                    common.message(_('%d record saved!') % len(result))
                 else:
-                    common.message(_('%d records saved!') % len(result),
-                            self.parent)
+                    common.message(_('%d records saved!') % len(result))
             return True
-        except Exception, exception:
+        except IOError, exception:
             common.warning(_("Operation failed!\nError message:\n%s") \
-                     % (exception[0],), self.parent, _('Error'))
+                     % (exception.faultCode,), _('Error'))
             return False
 
     def datas_read(self, ids, model, fields, context=None):
@@ -447,7 +446,7 @@ class WinExport(object):
         try:
             datas = rpc.execute('model', model,
                     'export_data', ids, fields, ctx)
-        except Exception, exception:
-            common.process_exception(exception, self.dialog)
+        except TrytonServerError, exception:
+            common.process_exception(exception)
             return []
         return datas
