@@ -1,23 +1,26 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from tryton.common import TRYTON_ICON
+from tryton.common import TRYTON_ICON, COLOR_SCHEMES
 import tryton.common as common
 from tryton.config import CONFIG
 import gtk
 import pango
 import gettext
+from tryton.exceptions import TrytonServerError
+from tryton.gui.window.nomodal import NoModal
+import tryton.rpc as rpc
 
 _ = gettext.gettext
 
 
-class WinForm(object):
+class WinForm(NoModal):
     "Form window"
 
-    def __init__(self, screen, parent, view_type='form', new=False,
-            many=False, context=None):
-
-        self.parent = parent
+    def __init__(self, screen, callback, view_type='form',
+            new=False, many=False, context=None):
+        NoModal.__init__(self)
         self.screen = screen
+        self.callback = callback
         self.context = context
         self.prev_view = self.screen.current_view
         self.screen.screen_container.alternate_view = True
@@ -31,13 +34,12 @@ class WinForm(object):
             self.screen.switch_view(view_type=view_type, context=context)
         if new and not switch_new:
             self.screen.new(context=self.context)
-        self.win = gtk.Dialog(_('Link'), parent,
-                gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT)
-        self.win.set_property('default-width', 760)
-        self.win.set_property('default-height', 500)
+        self.win = gtk.Dialog(_('Link'), self.parent,
+                gtk.DIALOG_DESTROY_WITH_PARENT)
         self.win.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.win.set_icon(TRYTON_ICON)
         self.win.set_has_separator(False)
+        self.win.connect('response', self.response)
 
         self.accel_group = gtk.AccelGroup()
         self.win.add_accel_group(self.accel_group)
@@ -55,7 +57,7 @@ class WinForm(object):
         if new and many:
             self.but_ok.add_accelerator('clicked',
                 self.accel_group, gtk.keysyms.Return,
-                gtk.gdk.CONTROL_MASK|gtk.gdk.SHIFT_MASK,
+                gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK,
                 gtk.ACCEL_VISIBLE)
 
             self.but_new = self.win.add_button(gtk.STOCK_NEW,
@@ -129,7 +131,8 @@ class WinForm(object):
             tooltips.set_tip(self.but_del, _('Delete selected record'))
             self.but_del.connect('clicked', self._sig_remove)
             img_del = gtk.Image()
-            img_del.set_from_stock('tryton-delete', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_del.set_from_stock('tryton-delete',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_del.set_alignment(0.5, 0.5)
             self.but_del.add(img_del)
             self.but_del.set_relief(gtk.RELIEF_NONE)
@@ -141,7 +144,8 @@ class WinForm(object):
             tooltips.set_tip(but_pre, _('Previous'))
             but_pre.connect('clicked', self._sig_previous)
             img_pre = gtk.Image()
-            img_pre.set_from_stock('tryton-go-previous', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_pre.set_from_stock('tryton-go-previous',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_pre.set_alignment(0.5, 0.5)
             but_pre.add(img_pre)
             but_pre.set_relief(gtk.RELIEF_NONE)
@@ -154,7 +158,8 @@ class WinForm(object):
             tooltips.set_tip(but_next, _('Next'))
             but_next.connect('clicked', self._sig_next)
             img_next = gtk.Image()
-            img_next.set_from_stock('tryton-go-next', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_next.set_from_stock('tryton-go-next',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_next.set_alignment(0.5, 0.5)
             but_next.add(img_next)
             but_next.set_relief(gtk.RELIEF_NONE)
@@ -166,7 +171,8 @@ class WinForm(object):
             tooltips.set_tip(but_switch, _('Switch'))
             but_switch.connect('clicked', self.switch_view)
             img_switch = gtk.Image()
-            img_switch.set_from_stock('tryton-fullscreen', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_switch.set_from_stock('tryton-fullscreen',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_switch.set_alignment(0.5, 0.5)
             but_switch.add(img_switch)
             but_switch.set_relief(gtk.RELIEF_NONE)
@@ -189,23 +195,19 @@ class WinForm(object):
 
         scroll.add(self.screen.screen_container.alternate_viewport)
 
-        width, height = self.screen.current_view.widget.size_request()
-        scroll.set_size_request(width, height + 30)
-        parent_width, parent_height = parent.get_size()
-        win_width, win_height = self.win.get_size()
-        self.widget_width = min(parent_width - 20, max(win_width, width + 20))
-        self.widget_height = min(parent_height - 60, height + win_height + 20)
-        self.win.set_default_size(self.widget_width, self.widget_height)
+        sensible_allocation = self.sensible_widget.get_allocation()
+        self.win.set_default_size(int(sensible_allocation.width * 0.9),
+            int(sensible_allocation.height * 0.9))
 
         if view_type == 'tree':
             self.screen.signal_connect(self, 'record-message', self._sig_label)
             self.screen.screen_container.alternate_viewport.connect(
                     'key-press-event', self.on_keypress)
 
+        self.register()
         self.win.show()
 
-        self.prev_window = self.screen.window
-        self.screen.window = self.win
+        common.center_window(self.win, self.parent, self.sensible_widget)
 
         self.screen.display()
         self.screen.current_view.set_cursor()
@@ -254,6 +256,7 @@ class WinForm(object):
         from tryton.gui.window.win_search import WinSearch
         domain = []
         context = rpc.CONTEXT.copy()
+        model_name = self.screen.model_name
 
         try:
             if self.wid_text.get_text():
@@ -261,25 +264,27 @@ class WinForm(object):
                         '%' + self.wid_text.get_text() + '%'), domain]
             else:
                 dom = domain
-            ids = rpc.execute('model', self.attrs['relation'], 'search', dom,
+            ids = rpc.execute('model', model_name, 'search', dom,
                     0, CONFIG['client.limit'], None, context)
-        except Exception, exception:
-            common.process_exception(exception, self.window)
+        except TrytonServerError, exception:
+            common.process_exception(exception)
             return False
-        if len(ids) != 1:
-            win = WinSearch(self.attrs['relation'], sel_multi=True, ids=ids,
-                    context=context, domain=domain, parent=self.window,
-                    views_preload=self.attrs.get('views', {}))
-            ids = win.run()
 
-        res_id = None
-        if ids:
-            res_id = ids[0]
-        self.screen.load(ids, modified=True)
-        self.screen.display(res_id=res_id)
-        if self.screen.current_view:
-            self.screen.current_view.set_cursor()
-        self.wid_text.set_text('')
+        def callback(ids):
+            res_id = None
+            if ids:
+                res_id = ids[0]
+            self.screen.load(ids, modified=True)
+            self.screen.display(res_id=res_id)
+            if self.screen.current_view:
+                self.screen.current_view.set_cursor()
+            self.wid_text.set_text('')
+
+        if len(ids) != 1:
+            WinSearch(model_name, callback, sel_multi=True,
+                ids=ids, context=context, domain=domain)
+        else:
+            callback(ids)
 
     def _sig_label(self, screen, signal_data):
         name = '_'
@@ -288,30 +293,30 @@ class WinForm(object):
         line = '(%s/%s)' % (name, signal_data[1])
         self.label.set_text(line)
 
-    def run(self):
+    def response(self, win, response_id):
         validate = False
         cancel_responses = (gtk.RESPONSE_CANCEL, gtk.RESPONSE_DELETE_EVENT)
-        while not validate:
-            response = self.win.run()
-            self.screen.current_view.set_value()
-            if (response in cancel_responses
-                    or self.screen.current_record is None):
-                break
+        self.screen.current_view.set_value()
+        if (response_id not in cancel_responses
+                and self.screen.current_record is not None):
             validate = self.screen.current_record.validate(
                 self.screen.current_view.get_fields())
             if not validate:
                 self.screen.current_view.set_cursor()
                 self.screen.display()
-                continue
-            if response == gtk.RESPONSE_ACCEPT:
+                return
+            if response_id == gtk.RESPONSE_ACCEPT:
                 self.new()
-                validate = False
+                return
         if (self.but_cancel
                 and self.screen.current_record
-                and response in cancel_responses):
+                and response_id in cancel_responses):
             self.screen.group.remove(self.screen.current_record, remove=True)
-            return False
-        return response not in cancel_responses
+            result = False
+        else:
+            result = response_id not in cancel_responses
+        self.destroy()
+        self.callback(result)
 
     def new(self):
         self.screen.new(context=self.context)
@@ -325,6 +330,12 @@ class WinForm(object):
             viewport.remove(viewport.get_child())
         self.screen.switch_view(view_type=self.prev_view.view_type)
         self.screen.signal_unconnect(self)
-        self.screen.window = self.prev_window
         self.win.destroy()
-        self.parent.present()
+        NoModal.destroy(self)
+
+    def show(self):
+        self.win.show()
+        common.center_window(self.win, self.parent, self.sensible_widget)
+
+    def hide(self):
+        self.win.hide()
